# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.db import models
from django.shortcuts import (
    get_object_or_404,
    render_to_response,
)
from django.template import RequestContext
from django.views.decorators.http import require_GET

from django_tables2 import RequestConfig

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)
from common.views import index

from memevent.models import (
    MemoryEventDetail,
    MemoryEventImage,
)
from memevent.tables import (
    DetailTable,
    ImageOverviewTable,
    ArchOverviewTable,
    EventDetailTable,
)


def _get_arches():
    arches = [x['image__arch'] for x in MemoryEventDetail.objects.filter(
        publish=True,
    ).values('image__arch').distinct()]

    return sorted(arches)


@BreadCrumb("Architecture Overview", parent=index)
@require_GET
def arch_overview(request, arch=None):

    arches = _get_arches()

    if arch is None:
        if len(arches) > 0:
            arch = arches[0]
        else:
            arch = "UNKNOWN"

    details = MemoryEventImage.objects.filter(
        publish=True,
        arch=arch,
    ).values(
        'id',
        'arch',
        'release',
        'variant',
        'build_number',
        'memoryeventdetail__event',
        'memoryeventdetail__run',
    ).annotate(
        pss_sum=models.Sum('memoryeventdetail__pss'),
    )

    _details = {}
    for detail in details:
        build_number = detail['build_number']
        image_id = detail['id']
        event = detail['memoryeventdetail__event']
        run = detail['memoryeventdetail__run']
        pss_sum = detail['pss_sum']
        arch = detail['arch']
        release = detail['release']
        variant = detail['variant']

        # ensure there are defaults
        image = _details.setdefault(build_number, {})
        image.setdefault(
            'events', {},
        ).setdefault(event, {}).setdefault('runs', {})

        image['id'] = image_id
        image['events'][event]['runs'][run] = pss_sum

    results = []
    # calculate averages
    for build_number, bn_data in _details.items():
        events = ""
        for event, ev_data in bn_data['events'].items():
            ev_data['average'] = 0
            values = ev_data['runs'].values()

            if len(values) > 0:
                ev_data['average'] = float(sum(values)) / len(values)

            events += "  {} ({:.2f}M)".format(event, ev_data['average'] / 1024)

        results.append(
            dict(
                id=bn_data['id'],
                build_number=build_number,
                events=events,
                release=release,
                variant=variant,
            ),
        )

    table = ArchOverviewTable(results)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        arch=arch,
        arches=arches,
        bread_crumb_trail=BreadCrumbTrail.leading_to(arch_overview),
    )

    return render_to_response(
        'memevent/arch_overview.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Image Overview",
    parent=arch_overview,
    needs=['image_id'],
)
@require_GET
def image_overview(request, image_id):

    image = get_object_or_404(MemoryEventImage, id=image_id)

    details = MemoryEventImage.objects.filter(
        publish=True,
        id=image_id,
    ).values(
        'id',
        'arch',
        'release',
        'variant',
        'build_number',
        'memoryeventdetail__event',
        'memoryeventdetail__run',
    ).annotate(
        pss_sum=models.Sum('memoryeventdetail__pss'),
    )

    table = ImageOverviewTable(details)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        image=image,
        bread_crumb_trail=BreadCrumbTrail.leading_to(
            image_overview,
            image_id=image_id,
        ),
    )

    return render_to_response(
        'memevent/image_overview.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Image Details",
    parent=image_overview,
    needs=['image_id'],
)
@require_GET
def image_detail(request, image_id):

    image = get_object_or_404(MemoryEventImage, id=image_id)

    details = MemoryEventDetail.objects.filter(
        publish=True,
        image=image,
    )

    table = DetailTable(details)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        image=image,
        bread_crumb_trail=BreadCrumbTrail.leading_to(
            image_detail,
            image_id=image_id,
        ),
    )

    return render_to_response(
        'memevent/image_detail.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Event Details",
    parent=image_detail,
    needs=['image_id', 'event'],
)
@require_GET
def event_detail(request, image_id, event):

    image = get_object_or_404(MemoryEventImage, id=image_id)

    details = MemoryEventDetail.objects.filter(
        publish=True,
        image=image,
        event=event,
    ).order_by('run')

    table = EventDetailTable(details)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        image=image,
        event=event,
        bread_crumb_trail=BreadCrumbTrail.leading_to(
            event_detail,
            image_id=image_id,
            event=event,
        ),
    )

    return render_to_response(
        'memevent/event_detail.html',
        data,
        RequestContext(request),
    )
