# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.management import call_command
from django.db import models
from django.template.loader import render_to_string

from common.plugin_helper import (
    Extension,
    KPI,
)

from smokeng.models import (
    SmokeResult,
    SmokeResultBug,
)
from smokeng.tables import SmokeKPITable


class SmokengPassRateKPI(KPI):
    """
    The original KPI entry that shows more data for each release.
    This class is used as the base class for the versions of the KPI
    that followed.

    """

    @classmethod
    def name(self):
        return "Smoke Testing Pass Rates"

    def importance(self):
        return self.IMPORTANCE_HIGH

    def _get_result_bug_count(self, build_number):
        qs = SmokeResultBug.objects.filter(
            publish=True,
            result__image__build_number=build_number,
        ).values(
            'result__image__release',
            'bug__bug_no',
        ).annotate(
            models.Count('bug__bug_no'),
        ).values(
            'bug__bug_no',
            'result__image__release',
            'bug__bug_no__count',
        )

        result = dict()
        for bug in qs:
            result.setdefault(
                bug['result__image__release'],
                0,
            )
            result[bug['result__image__release']] += 1

        return result

    def _get_result_data(self):
        qs = SmokeResult.objects.filter(
            publish=True,
        ).values(
            'image__arch',
            'image__release',
            'image__build_number',
        ).annotate(
            avg_pass_rate=models.Avg('pass_rate'),
        ).order_by('-image__build_number')

        return qs

    def _get_entry_tree(self):
        entries = self._get_result_data()

        entry_tree = {}
        for entry in entries:
            build = entry_tree.setdefault(
                entry['image__release'],
                {},
            ).setdefault(
                entry['image__arch'],
                {},
            ).setdefault(
                entry['image__build_number'],
                {},
            )

            build.setdefault('pass_rates', []).append(entry['avg_pass_rate'])

        return entry_tree

    def _calculate_pass_rates(self):

        entry_tree = self._get_entry_tree()
        TREND_ENTRIES = 7

        results = {}
        for release, arches in entry_tree.items():
            #print("release: {}, arches: {}".format(release, arches))
            for arch_name, arch in arches.items():
                res_arch = results.setdefault(
                    release,
                    {},
                ).setdefault(
                    arch_name,
                    {},
                )
                keys = arch.keys()
                item_count = min(len(keys), TREND_ENTRIES)
                for build in sorted(keys, reverse=True)[:item_count]:
                    res_arch.setdefault(
                        'latest_build_number',
                        build
                    )

                    arch_build = arch[build]
                    arch_build['pass_rate'] = 0
                    if len(arch_build['pass_rates']) > 0:
                        pr_sum = sum(arch_build['pass_rates'])
                        pr_len = len(arch_build['pass_rates'])
                        arch_build['pass_rate'] = pr_sum / pr_len

                    res_arch.setdefault('pass_rate', arch_build['pass_rate'])

                    if build > res_arch['latest_build_number']:
                        res_arch['latest_build_number'] = build
                        res_arch['pass_rate'] = arch_build['pass_rate']

                    res_arch.setdefault('data', []).append(
                        dict(
                            release=release,
                            arch=arch_name,
                            build_number=build,
                            pass_rate=arch_build['pass_rate'],
                        )
                    )

        return results

    def render(self):

        results = self._calculate_pass_rates()

        tables = []
        for release in sorted(results.keys(), reverse=True):
            arches = results[release]
            entries = []

            for arch in sorted(arches.keys()):
                data = arches[arch]
                entries.append(
                    dict(
                        release=release,
                        arch=arch,
                        build_number=data['latest_build_number'],
                        pass_rate=data['pass_rate'] * 100,
                    )
                )

            tables.append(
                dict(
                    release=release,
                    table=SmokeKPITable(
                        entries,
                        sortable=False,
                        prefix=release,
                    ),
                )
            )

        retval = render_to_string(
            'smokeng/kpi_pass_rate.html',
            dict(
                tables=tables,
            ),
        )

        return retval


class SmokengSimplePassRateKPI(SmokengPassRateKPI):
    """
    A simplified version of SmokengPassRateKPI. This KPI returns only
    a pass-rate for each release.

    This class' render() method doesn't distinguish between image
    build_numbers and thus isn't necessarily comparing apples to apples.

    """

    def render(self):

        results = self._calculate_pass_rates()

        entries = []
        for release in sorted(results.keys(), reverse=True):
            arches = results[release]

            arch_passrates = []
            for arch in sorted(arches.keys()):
                data = arches[arch]
                arch_passrates.append(data['pass_rate'])

            pass_rate = 0.0

            if len(arch_passrates) > 0:
                pass_rate = sum(arch_passrates) / len(arch_passrates)

            entries.append(
                dict(
                    release=release,
                    pass_rate=pass_rate * 100,
                )
            )

        retval = render_to_string(
            'smokeng/kpi_simple_pass_rate.html',
            dict(
                entries=entries,
            ),
        )

        return retval


class SmokengPassRateV3KPI(SmokengPassRateKPI):
    """
    A very simple KPI entry with an average pass-rate for each release.

    This KPI determines the latest build number that there are results for
    and shows an average of those results for each release.

    To identify missing images a list of architectures used to calculate
    the pass-rate for each release are listed.

    A count of bugs filed by the QA Team is also included.

    """

    def _get_latest_build_number(self, results):

        BUILD_NUMBER_DIGITS = 8  # YYYYMMDD

        build_numbers = set()

        for release, arches in results.items():
            for arch, data in arches.items():
                build_numbers.add(data['latest_build_number'])

        return max(build_numbers)[:BUILD_NUMBER_DIGITS]

    def render(self):

        results = self._calculate_pass_rates()

        expected_build = self._get_latest_build_number(results)
        bug_counts = self._get_result_bug_count(expected_build)

        skipped_releases = ['quantal', 'raring']
        entries = []
        for release in sorted(results.keys(), reverse=True):
            if release in skipped_releases:
                continue
            found_arches = []
            arches = results[release]

            arch_passrates = []
            for arch in sorted(arches.keys()):
                data = arches[arch]
                build_no = data['latest_build_number'][:len(expected_build)]
                if build_no == expected_build:
                    arch_passrates.append(data['pass_rate'])
                    found_arches.append(arch)

            pass_rate = 0.0

            if len(arch_passrates) > 0:
                pass_rate = sum(arch_passrates) / len(arch_passrates)

            bug_count = bug_counts.get(release, 0)
            entries.append(
                dict(
                    release=release,
                    pass_rate=pass_rate * 100,
                    found_arches=found_arches,
                    bug_count=bug_count,
                )
            )

        retval = render_to_string(
            'smokeng/kpi_simple_pass_rate.html',
            dict(
                entries=entries,
                build_number=expected_build,
                help_info="""The average pass_rate for each architecture
                (i.e. "amd64", "i386", "armhf") image is calculated.  These
                averages are averaged for each release being tracked.  Hover
                over the pass rate for each release for the number of bugs
                reported.""",
            ),
        )

        return retval


class SmokengExtension(Extension):

    def contribute_to_kpis(self, kpi_mgr):
        return [
            SmokengPassRateV3KPI,
        ]

    def pull_data(self):
        call_command('jenkins_pull_smokeng')

    @classmethod
    def name(cls):
        return cls.__name__


extension = SmokengExtension()
