# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.shortcuts import (
    render_to_response,
    get_object_or_404,
)
from django_tables2 import RequestConfig
from django.template import RequestContext
from django.views.decorators.http import require_GET

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from common.views import index

from smokeng.models import (
    SmokeImage,
    SmokeResult,
    SmokeResultBug,
)
from smokeng.tables import (
    SmokeImageTable,
    SmokeResultTable,
)


def _get_releases():
    return [
        image['release'] for image in SmokeImage.objects.values(
            'release',
        ).order_by('-release').distinct().exclude(publish=False)
    ]


def get_totals(
    results,
    release,
):
    totals = {}
    passrates = {}

    bugs = _get_bugs(release)

    for result in results:
        _append_totals(totals, result)
        if result['total_count'] == 0:
            passrate = 0
        else:
            passrate = float(result['pass_count']) / result['total_count']

        passrates.setdefault(result['image__id'], []).append(passrate)

    for image, passrate in passrates.iteritems():
        totals[image]['pass_rate'] = \
            sum(passrates[image]) / len(passrates[image])
        totals[image]['pass_rate_pct'] = \
            totals[image]['pass_rate'] * 100

    for bug in bugs.iterator():
        if bug.result.image.id in totals:
            if bug.bug.bug_no not in totals[bug.result.image.id]['bugs']:
                totals[bug.result.image.id]['bugs'].append(bug.bug.bug_no)

    return totals


def _get_bugs(release):

    bugs = SmokeResultBug.objects.filter(
        result__image__release=release,
    ).select_related()

    return bugs


def _append_totals(totals, result):
    key = result['image__id']

    if key not in totals:
        totals[key] = dict(
            build_number=result['image__build_number'],
            release=result['image__release'],
            total_count=0,
            pass_count=0,
            fail_count=0,
            error_count=0,
            bugs=[],
            name=result['name'],
        )

        if 'image__variant' in result:
            totals[key]['variant'] = result['image__variant']

        if 'image__id' in result:
            totals[key]['image_id'] = result['image__id']

        if 'image__arch' in result:
            totals[key]['arch'] = result['image__arch']

        if 'ran_at' in result:
            totals[key]['ran_at'] = result['ran_at']

    totals[key]['total_count'] += result['total_count']
    totals[key]['pass_count'] += result['pass_count']
    totals[key]['error_count'] += result['error_count']
    totals[key]['fail_count'] += result['fail_count']


@BreadCrumb("{release} smoke testing", parent=index, needs=['release'])
@require_GET
def overview(request, release=None):

    if release is None:
        releases = _get_releases()
        release = releases[0]

    results = SmokeResult.objects.filter(
        publish=True,
        image__release=release,
    ).values(
        'image__build_number',
        'image__variant',
        'image__arch',
        'image__id',
        'image__release',
        'name',
        'ran_at',
    ).annotate(
        pass_count=models.Sum('pass_count'),
        fail_count=models.Sum('fail_count'),
        error_count=models.Sum('error_count'),
        total_count=models.Sum('total_count'),
    ).values(
        'image__build_number',
        'image__variant',
        'image__arch',
        'image__id',
        'image__release',
        'name',
        'ran_at',
        'pass_count',
        'fail_count',
        'error_count',
        'total_count',
    )

    totals = get_totals(results, release)

    table = SmokeImageTable(totals.values())

    RequestConfig(request, paginate=None).configure(table)

    data = {
        'table': table,
        'release': release,
        'releases': _get_releases(),
        'active': release,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            overview,
            release=release,
        )
    }

    return render_to_response(
        'smokeng/overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Image {build_number}",
    parent=overview,
    needs=['release', 'image_id'],
)
@require_GET
def image_overview(request, release, image_id):

    image = get_object_or_404(SmokeImage, id=image_id)
    summary = image.summary

    table = SmokeResultTable(summary['results'])
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'image': image,
        'summary': summary,
        'build_number': image.build_number,
        'release': release,
        'table': table,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_overview,
            release=release,
            image_id=image_id,
            build_number=image.build_number,
        ),
    }
    return render_to_response(
        'smokeng/image_overview.html',
        data,
        RequestContext(request),
    )
